# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-10-07 19:09
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djenga', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='managementcommand',
            name='last_run',
            field=models.DateTimeField(help_text='The last time this command was run', null=True),
        ),
        migrations.AlterField(
            model_name='managementcommand',
            name='last_success',
            field=models.DateTimeField(help_text='The last time this command was run successfully', null=True),
        ),
        migrations.AlterField(
            model_name='managementcommand',
            name='name',
            field=models.CharField(db_index=True, help_text='The full path of the command', max_length=128),
        ),
        migrations.AlterField(
            model_name='managementcommand',
            name='status',
            field=models.CharField(help_text='The current status of the command', max_length=32),
        ),
    ]
